@@ -16,8 +16,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 /* #define DEBUG */
 
 #include <gtk/gtk.h>
@@ -133,7 +134,7 @@ int main(int argc, char *argv[])
 	GList *filenames = NULL, *projectfiles=NULL;
 	Tbfwin *firstbfwin;
 #ifndef NOSPLASH
-	GtkWidget *splash_window;
+	GtkWidget *splash_window = NULL;
 #endif /* #ifndef NOSPLASH */
 
 #ifdef ENABLE_NLS                                                               
@@ -175,9 +176,11 @@ int main(int argc, char *argv[])
 	}
 #endif /* WITH_MSG_QUEUE */
 #ifndef NOSPLASH
-	/* start splash screen somewhere here */
-	splash_window = start_splash_screen();
-	splash_screen_set_label(_("parsing highlighting file..."));
+	if (main_v->props.show_splash_screen) {
+		/* start splash screen somewhere here */
+		splash_window = start_splash_screen();
+		splash_screen_set_label(_("parsing highlighting file..."));
+	}
 #endif /* #ifndef NOSPLASH */
 
 	{
@@ -188,14 +191,14 @@ int main(int argc, char *argv[])
 	rcfile_parse_global_session();
 	rcfile_parse_highlighting();
 #ifndef NOSPLASH
-	splash_screen_set_label(_("compiling highlighting patterns..."));
+	if (main_v->props.show_splash_screen) splash_screen_set_label(_("compiling highlighting patterns..."));
 #endif /* #ifndef NOSPLASH */
 	hl_init();
 	filebrowserconfig_init();
 	filebrowser_filters_rebuild();
 	autoclosing_init();
 #ifndef NOSPLASH
-	splash_screen_set_label(_("parsing custom menu file..."));
+	if (main_v->props.show_splash_screen) splash_screen_set_label(_("parsing custom menu file..."));
 #endif /* #ifndef NOSPLASH */
 	rcfile_parse_custom_menu(FALSE,FALSE);
 	main_v->tooltips = gtk_tooltips_new();
@@ -207,7 +210,7 @@ int main(int argc, char *argv[])
 	}
 #endif /* WITH_MSG_QUEUE */
 #ifndef NOSPLASH
-	splash_screen_set_label(_("creating main gui..."));
+	if (main_v->props.show_splash_screen) splash_screen_set_label(_("creating main gui..."));
 #endif /* #ifndef NOSPLASH */
 
 	/* create the first window */
@@ -216,9 +219,9 @@ int main(int argc, char *argv[])
 	firstbfwin->bookmarkstore = main_v->bookmarkstore;
 	main_v->bfwinlist = g_list_append(NULL, firstbfwin);
 	gui_create_main(firstbfwin,filenames);
-   bmark_reload(firstbfwin);
+	bmark_reload(firstbfwin);
 #ifndef NOSPLASH
-	splash_screen_set_label(_("showing main gui..."));
+	if (main_v->props.show_splash_screen) splash_screen_set_label(_("showing main gui..."));
 #endif /* #ifndef NOSPLASH */
 	/* set GTK settings, must be AFTER the menu is created */
 	{
@@ -231,9 +234,46 @@ int main(int argc, char *argv[])
 	}
 
 	gui_show_main(firstbfwin);
-	if (firstbfwin->session->view_html_toolbar && main_v->globses.quickbar_items == NULL) {
-		info_dialog(firstbfwin->main_window, _("Bluefish tip:"), _("This message is shown since you do not have any items in the Quickbar.\n\nIf you right-click a button in the HTML toolbars you can add buttons to the Quickbar."));
+	if (firstbfwin->session->view_html_toolbar && 
+         main_v->globses.quickbar_items == NULL && 
+         main_v->props.show_quickbar_tip == TRUE) 
+   {	
+	   GtkWidget *dialog, *hbox, *image, *vbox, *label, *button, *show_again;
+	   
+	   dialog = gtk_dialog_new_with_buttons ("", 
+	                                         GTK_WINDOW (firstbfwin->main_window),
+	                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                         NULL);
+		button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+		
+		hbox = gtk_hbox_new (FALSE, 12);
+		gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox);
+		image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+		gtk_misc_set_alignment (GTK_MISC (image), 0 ,0);
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		vbox = gtk_vbox_new (FALSE, 0);
+		gtk_box_pack_start_defaults (GTK_BOX (hbox), vbox);
+		label = gtk_label_new (NULL);
+		gtk_label_set_markup (GTK_LABEL (label), 
+									 _("<span weight=\"bold\" size=\"larger\">Bluefish Tip:</span>\n\nThis message is shown if you do not have any items in the Quickbar.\n\nTo add buttons to the Quickbar, right click on a button in the HTML toolbars.\n"));
+		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+
+	   show_again = gtk_check_button_new_with_mnemonic (_("_Don't show this dialog again."));
+	   gtk_box_pack_start (GTK_BOX (vbox), show_again, FALSE, FALSE, 12);
+	   
+	   gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
+		gtk_widget_grab_focus (button);
+	   gtk_dialog_run (GTK_DIALOG (dialog));
+	   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (show_again))) {
+	      main_v->props.show_quickbar_tip = FALSE;
+	   }
+	   gtk_widget_destroy (dialog);
 	}
+	
 	if (projectfiles) {
 		GList *tmplist = g_list_first(projectfiles);
 		while (tmplist) {
@@ -243,12 +283,12 @@ int main(int argc, char *argv[])
 	}
 	
 #ifndef NOSPLASH
-	flush_queue();
-	{
+	if (main_v->props.show_splash_screen) {
 		static struct timespec const req = { 0, 10000000};
+		flush_queue();
 		nanosleep(&req, NULL);
+		gtk_widget_destroy(splash_window);
 	}
-	gtk_widget_destroy(splash_window);
 #endif /* #ifndef NOSPLASH */
 	DEBUG_MSG("main, before gtk_main()\n");
 	gtk_main();
