@@ -2,7 +2,7 @@
  * bluefish.h - global prototypes
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2004 Olivier Sessink
+ * Copyright (C) 1999-2005 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 /* if you define DEBUG here you will get debug output from all Bluefish parts */
 /* #define DEBUG */
 
@@ -133,7 +134,7 @@ typedef struct {
 	GtkWidget *tab_eventbox;
 	GtkWidget *tab_menu;
 	GtkTextBuffer *buffer;
-	gboolean in_paste_operation;
+	gpointer paste_operation;
 	gint last_rbutton_event; /* index of last 3rd button click */
 	Tfiletype *hl; /* filetype & highlighting set to use for this document */
 	gint need_highlighting; /* if you open 10+ documents you don't need immediate highlighting, just set this var, and notebook_switch() will trigger the actual highlighting when needed */
@@ -149,18 +150,20 @@ typedef struct {
 } Tdocument;
 
 typedef struct {
-	gint view_html_toolbar;				/* view html toolbar */
-	gint view_custom_menu;			/* view custom menubar */
-	gint view_main_toolbar;				/* view main toolbar */
-	gint view_left_panel;  	/* view filebrowser/functionbrowser etc. */
+#ifndef NOSPLASH
+	gint show_splash_screen;  /* show splash screen at startup */
+#endif /* #ifndef NOSPLASH */
+	gint show_quickbar_tip;
 	gint view_line_numbers; /* view line numbers on the left side by default */
 	gint filebrowser_show_hidden_files;
 	gint filebrowser_show_backup_files;
 	gint filebrowser_two_pane_view; /* have one or two panes in the filebrowser */
+	gint filebrowser_focus_follow; /* have the directory of the current document in focus */
 	gchar *filebrowser_unknown_icon;
 	gchar *filebrowser_dir_icon;
 	gchar *editor_font_string;		/* editor font */
 	gint editor_tab_width;	/* editor tabwidth */
+	gint editor_smart_cursor;
 	gint editor_indent_wspaces; /* indent with spaces, not tabs */
 	gchar *tab_font_string;		/* notebook tabs font */
 	GList *browsers; /* browsers array */
@@ -170,6 +173,9 @@ typedef struct {
 	GList *cmenu_replace; /* custom menu replaces */
 	gint highlight_num_lines_count; /* number of lines to highlight in continous highlighting */	
 	gint defaulthighlight;		/* highlight documents by default */
+#ifdef HAVE_PCRE_UTF8
+	gint highlight_utf8;    /* enable PCRE UTF-8 support */
+#endif /* HAVE_PCRE_UTF8 */
 	GList *filetypes; /* filetypes for highlighting and filtering */
 	gint numcharsforfiletype; /* maximum number of characters in the file to use to find the filetype */
 	GList *filefilters; /* filebrowser.c filtering */
@@ -178,6 +184,7 @@ typedef struct {
 	gint transient_htdialogs;  /* set html dialogs transient ro the main window */
 	gint restore_dimensions; /* use the dimensions as used the previous run */
 	gint left_panel_width; 	/* width of filelist */
+	gint left_panel_left; /* 1 = left, 0 = right */
 	gint max_recent_files;	/* length of Open Recent list */
 	gint max_dir_history;	/* length of directory history */
 	gint backup_file; 			/* wheather to use a backup file */
@@ -198,14 +205,15 @@ typedef struct {
 	gchar *newfile_default_encoding; /* if you open a new file, what encoding will it use */
 	GList *encodings; /* all encodings you can choose from */
 	gint auto_set_encoding_meta; /* auto set metatag for the encoding */
-	gint auto_update_meta; /* auto update author and generator meta tags on save */
+	gint auto_update_meta_author; /* auto update author meta tag on save */
+	gint auto_update_meta_date; /* auto update date meta tag on save */
+	gint auto_update_meta_generator; /* auto update generator meta tag on save */
 	gint encoding_search_Nbytes; /* number of bytes to look for the encoding meta tag */
 	GList *outputbox; /* all outputbox commands */
 	gint ext_browsers_in_submenu;
 	gint ext_commands_in_submenu;
 	gint ext_outputbox_in_submenu;
 	GList *reference_files; /* all reference files */
-	gint bookmarks_sort; /* 0=no sort, 1=sort */
 	gint bookmarks_default_store; /* 0= temporary by default, 1= permanent by default */
 	gint bookmarks_filename_mode; /* 0=FULLPATH, 1=DIR FROM BASE 2=BASENAME */
 	gint document_tabposition;
@@ -268,6 +276,12 @@ typedef struct {
 	GList *bmarks;
 	GList *recent_files;
 	GList *recent_dirs;
+	gchar *opendir;
+	gchar *savedir;
+   gint view_html_toolbar;	/* view html toolbar */
+   gint view_custom_menu;  /* view custom menubar */
+   gint view_main_toolbar;	/* view main toolbar */
+   gint view_left_panel;	/* view filebrowser/functionbrowser etc. */
 } Tsessionvars;
 
 typedef struct {
@@ -278,10 +292,6 @@ typedef struct {
 	gchar *webdir;
 	gchar *template;
 	gpointer editor;
-	gint view_main_toolbar;
-	gint view_left_panel;
-	gint view_custom_menu;
-	gint view_html_toolbar;
 	gint word_wrap;
 	Tsessionvars *session;
 	GtkTreeStore *bookmarkstore; /* project bookmarks */
