@@ -1,5 +1,5 @@
 /* Bluefish HTML Editor
- * filebrowser.c the filebrowser
+ * filebrowser.c - the filebrowser
  *
  * Copyright (C) 2002-2003 Olivier Sessink
  *
@@ -15,28 +15,68 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 /* #define DEBUG */
 
+/* ******* NEW FILEBROWSER DESIGN ********
+I'm thinking about a new design for the filebrowser, the 
+code is too complicated right now. For now I'll just write some thoughts, so 
+after we have the next stable releae we can implement it.
+
+we'll drop the one-pane-view, we'll only have the files and directories separate
+
+the public API:
+--------------
+GtkWidget *fb2_init(Tbfwin *bfwin);
+void fb2_cleanup(Tbfwin *bfwin);
+void fb2_set_basedir(Tbfwin *bfwin, gchar *basedir);
+void fb2_focus_document(Tbfwin *bfwin, Tdocument *doc);
+--------------
+
+in the treemodel for the directories, we'll have two columns. A visible column (with the name), 
+and a column with the full path,  so if a click on an item is done, it is very easy to see which 
+full path corresponds to that item.
+
+to make 'focus document' very easy, each document can have a GtkTreeIter 
+pointing to the directory where the file located. If it does not have the treeiter, the 
+document has not been focused before (so it is not required to have a treeiter for 
+every document).
+
+the most difficult thing to code now is when we for example open a new file, we'll have to 
+find which directory item corresponds to that filename, and if it is not yet there, we'll 
+have to find where we should add it. Same for a document that has not been focused before.
+A possibility to do this is to have a hashtable with TreeIters to each directory that is already 
+in the tree. If you then want to open some directory, you check the full path in the hashtable, 
+if not exists you remove the last directory component, check again etc. until you have found a
+position in the tree where you can add things.
+
+USER INTERFACE NOTES:
+
+each directory should be expandable by default (so have a dummy item) unless we know there are 
+no subdirectories (and not the other way around like it is right now)
+
+*/
 #include <gtk/gtk.h>
-#include <sys/types.h> /* stat() getuid */
-#include <sys/stat.h> /* stat() */
-#include <unistd.h> /* stat() getuid */
-#include <string.h> /*strchr() */
-#include <stdio.h> /* rename() */	
-#include <stdlib.h> /* atoi() */
+#include <sys/types.h>	/* stat() getuid */
+#include <sys/stat.h>	/* stat() */
+#include <unistd.h>		/* stat() getuid */
+#include <string.h>		/*strchr() */
+#include <stdio.h>		/* rename() */	
+#include <stdlib.h>		/* atoi() */
 
 #include "bluefish.h"
 
-#include "document.h"
-#include "gtk_easy.h" /* *_dialog(), flush_queue() */
-#include "bf_lib.h"
-#include "stringlist.h" /* count_array() */
 #include "filebrowser.h"
-#include "gui.h" /* statusbar_message() */
-#include "image.h" /* image_insert_from_filename() */
-#include "menu.h" /* translation */
+#include "bf_lib.h"
+#include "document.h"
+#include "gtk_easy.h"	/* *_dialog(), flush_queue() */
+#include "gui.h"			/* statusbar_message() */
+#include "image.h" 		/* image_insert_from_filename() */
+#include "menu.h" 		/* translation */
+#include "project.h" 	/* project_open_from_file() */
+#include "stringlist.h" /* count_array() */
 
 /*#define DEBUG_SORTING
 #define DEBUG_FILTER
@@ -93,6 +133,7 @@ typedef struct {
 	GtkListStore *store2;
 	GtkWidget *dirmenu;
 	GtkWidget *showfulltree;
+	GtkWidget *focus_follow;
 	GList *dirmenu_entries;
 	gchar *last_opened_dir; /* SHOULD end on a '/' */
 	gboolean last_popup_on_dir;
@@ -171,7 +212,11 @@ static gchar *return_filename_from_path(Tfilebrowser *filebrowser, GtkTreeModel
 	gchar *retval = NULL, *tmp;
 	gboolean showfulltree = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filebrowser->showfulltree));
 	GtkTreePath *path = gtk_tree_path_copy(thispath);
-
+	DEBUG_MSG("return_filename_from_path, ");
+	DEBUG_DUMP_TREE_PATH(path);
+	if (!path) {
+		return NULL;
+	}
 	while (valid) {
 		gchar *name = NULL, *tmpstr;
 		GtkTreeIter iter;
@@ -815,12 +860,14 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 		filebrowser->last_opened_dir = ending_slash(dirname);
 		populate_dir_history(filebrowser,FALSE, filebrowser->last_opened_dir);
 		DEBUG_MSG("build_tree_from_path, after ending slash\n");
-		
+
+		/* we need to add an empty item to each dir so it can be expanded */		
 		direntrylist = return_dir_entries(filebrowser,dirname);
 		DEBUG_MSG("build_tree_from_path, after return_dir_entries\n");
 		tmplist = g_list_first(direntrylist);
 		while (tmplist) {
 			Tdir_entry *entry = (Tdir_entry *)tmplist->data;
+			DEBUG_MSG("build_tree_from_path, entry=%s\n",entry->name);
 			if (entry->type == TYPE_DIR) {
 				GtkTreeIter tmp;
 				tmp = add_tree_item(&iter, filebrowser, entry->name, TYPE_DIR, NULL);
@@ -838,6 +885,10 @@ static GtkTreePath *build_tree_from_path(Tfilebrowser *filebrowser, const gchar
 		DEBUG_MSG("build_tree_from_path, freed dirname\n");
 		free_dir_entries(direntrylist);
 		DEBUG_MSG("build_tree_from_path, cleaned direntrylist\n");
+		/* if the toplevel is an empty directory, it needs a '.' item as well */
+		if (!gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter)) {
+			add_tree_item(&iter, filebrowser, ".", TYPE_FILE, NULL);
+		}
 	}
 	return gtk_tree_model_get_path(GTK_TREE_MODEL(filebrowser->store),&iter);
 }
@@ -1044,6 +1095,8 @@ static void create_file_or_dir_win(Tfilebrowser *filebrowser, gint is_file) {
 	GtkTreeIter iter;
 	gchar *basedir = NULL;
 	selection = filebrowser_get_path_from_selection(GTK_TREE_MODEL(filebrowser->store),GTK_TREE_VIEW(filebrowser->tree),&iter);
+	DEBUG_MSG("create_file_or_dir_win, selection ");
+	DEBUG_DUMP_TREE_PATH(selection);
 	if (filebrowser->store2 && !selection) {
 		/* there is no selection, we'll use the current last used dir */
 		if (filebrowser->last_opened_dir) {
@@ -1054,6 +1107,7 @@ static void create_file_or_dir_win(Tfilebrowser *filebrowser, gint is_file) {
 		gchar *tmp;
 		if (filebrowser->store2 || gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter)) {
 			/* the selection does point to a directory, not a file */
+			DEBUG_MSG("create_file_or_dir_win, is a directory: filebrowser->store2=%p, tree_model_iter_has_child=%d\n",filebrowser->store2 ,gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store), &iter));
 		} else {
 			/* the selection points to a file, get the parent */
 			GtkTreeIter parent;
@@ -1127,6 +1181,8 @@ static void handle_activate_on_file(Tfilebrowser *filebrowser, gchar *filename)
 		gchar *relfilename = create_relative_link_to(filebrowser->bfwin->current_document->filename, filename);
 		image_insert_from_filename(filebrowser->bfwin,relfilename);
 		g_free(relfilename);
+	} else if (strcmp(ft->type, "bfproject") == 0) {
+		project_open_from_file(filebrowser->bfwin, filename);
 	} else {
 		DEBUG_MSG("handle_activate_on_file, file %s is not-editable, do something special now?\n",filename);
 	}
@@ -1187,10 +1243,10 @@ static void filebrowser_rpopup_rename(Tfilebrowser *filebrowser) {
 			/* If an error occurs, doc_save takes care of notifying the user.
 			 * Currently, nothing is done here.
 			 */	
-			doc_save(tmpdoc, 1, 1);
+			doc_save(tmpdoc, TRUE, TRUE, FALSE);
 		} else {
 			/* Promt user, "File/Move To"-style. */
-			newfilename = ask_new_filename(filebrowser->bfwin,oldfilename, 1);
+			newfilename = ask_new_filename(filebrowser->bfwin, oldfilename, oldfilename, TRUE);
 			if (newfilename) {
 				gchar *old_OnDiEn, *new_OnDiEn; /* OnDiskEncoding */
 				old_OnDiEn = get_filename_on_disk_encoding(oldfilename);
@@ -1244,8 +1300,8 @@ static void filebrowser_rpopup_delete(Tfilebrowser *filebrowser) {
 		gchar *buttons[] = {GTK_STOCK_CANCEL, GTK_STOCK_DELETE, NULL};
 		gchar *label;
 		gint retval;
-		label = g_strdup_printf(_("Are you sure you want to\ndelete \"%s\" ?"), filename);
-		retval = multi_query_dialog(filebrowser->bfwin->main_window,label, _("The file will be permanently deleted."), 0, 0, buttons);
+		label = g_strdup_printf(_("Are you sure you want to delete \"%s\" ?"), filename);
+		retval = multi_query_dialog(filebrowser->bfwin->main_window,label, _("If you delete this file, it will be permanently lost."), 0, 0, buttons);
 		g_free(label);
 		if (retval == 1) {
 			gchar *errmessage = NULL;
@@ -1397,7 +1453,7 @@ static void filebrowser_rpopup_sbf_toggled_lcb(GtkWidget *widget, Tfilebrowser *
 static GtkItemFactoryEntry filebrowser_dirmenu_entries[] = {
 #ifdef EXTERNAL_GREP
 #ifdef EXTERNAL_FIND	
-	{ N_("/Open _Advanced"),			NULL,	filebrowser_rpopup_action_lcb,		7,	"<Item>" },
+	{ N_("/Open _Advanced..."),			NULL,	filebrowser_rpopup_action_lcb,		7,	"<Item>" },
 #endif
 #endif
 	
@@ -1487,6 +1543,7 @@ static gboolean filebrowser_button_press_lcb(GtkWidget *widget, GdkEventButton *
 			gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(filebrowser->tree), event->x, event->y, &path, NULL, NULL, NULL);
 			if (path) {
 				gtk_tree_model_get_iter(GTK_TREE_MODEL(filebrowser->store),&iter,path);
+				gtk_tree_path_free(path);
 				if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(filebrowser->store),&iter)) {
 					/* it is a directory */
 					menu = filebrowser_rpopup_create_menu(filebrowser, TRUE);
@@ -1533,8 +1590,6 @@ static gboolean filebrowser_button_press_lcb(GtkWidget *widget, GdkEventButton *
 						g_free(tmp);
 						g_free(dir);
 					}
-	/*				gtk_tree_path_free(path);
-					return FALSE;*/
 				} else if (event->type == GDK_2BUTTON_PRESS) {
 					GdkRectangle r;
 					gchar *filename = return_filename_from_path(filebrowser,GTK_TREE_MODEL(filebrowser->store),path);
@@ -1543,11 +1598,11 @@ static gboolean filebrowser_button_press_lcb(GtkWidget *widget, GdkEventButton *
 					handle_activate_on_file(filebrowser,filename);
 					g_free(filename);
 					gtk_tree_view_scroll_to_point(GTK_TREE_VIEW(filebrowser->tree),-1,r.y);
+					gtk_tree_path_free (path);
 					return TRUE;
 				}
 			}
 			gtk_tree_path_free(path);
-/*			return TRUE;*/
 		}
 	}
 	return FALSE; /* pass the event on */
@@ -1816,6 +1871,10 @@ static void populate_dir_history(Tfilebrowser *filebrowser,gboolean firsttime, g
 	gtk_option_menu_set_history(GTK_OPTION_MENU(filebrowser->dirmenu), 0);
 }
 
+static void focus_follow_toggled_lcb(GtkToggleButton *togglebutton, Tbfwin *bfwin) {
+	main_v->props.filebrowser_focus_follow = togglebutton->active;
+}
+
 /**
  * filebrowsel_init:
  * @bfwin: #Tbfwin*
@@ -1853,7 +1912,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 	filebrowser->store = gtk_tree_store_new (N_COLUMNS,GDK_TYPE_PIXBUF,G_TYPE_STRING);
 	filebrowser->tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(filebrowser->store));
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(filebrowser->tree), FALSE);	
-
+	/*gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(filebrowser->tree), TRUE);*/
 	{
 		GtkCellRenderer *renderer;
 		GtkTreeViewColumn *column;
@@ -1893,7 +1952,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		filebrowser->store2 = gtk_list_store_new (N_COLUMNS,GDK_TYPE_PIXBUF,G_TYPE_STRING);
 		filebrowser->tree2 = gtk_tree_view_new_with_model(GTK_TREE_MODEL(filebrowser->store2));
 		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(filebrowser->tree2), FALSE);
-
+		/*gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(filebrowser->tree2), TRUE);*/
 		renderer = gtk_cell_renderer_pixbuf_new();
 		column = gtk_tree_view_column_new ();
 		gtk_tree_view_column_pack_start (column, renderer, FALSE);
@@ -1917,6 +1976,7 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		gchar *buildfrom = NULL;
 		
 		filebrowser->showfulltree = checkbut_with_value(_("Show full tree"), FALSE);
+		filebrowser->focus_follow = checkbut_with_value(_("Follow focus"), main_v->props.filebrowser_focus_follow);
 		if (bfwin->project && bfwin->project->basedir && strlen(bfwin->project->basedir)>2) {
 			buildfrom = bfwin->project->basedir;
 		} else {
@@ -1981,7 +2041,9 @@ GtkWidget *filebrowser_init(Tbfwin *bfwin) {
 		gtk_adjustment_set_value(GTK_ADJUSTMENT(adj), GTK_ADJUSTMENT(adj)->lower);*/
 		
 		gtk_box_pack_start(GTK_BOX(vbox), filebrowser->showfulltree, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(vbox), filebrowser->focus_follow, FALSE, FALSE, 0);
 		g_signal_connect(G_OBJECT(filebrowser->showfulltree), "toggled", G_CALLBACK(showfulltree_toggled_lcb), bfwin);
+		g_signal_connect(G_OBJECT(filebrowser->focus_follow), "toggled", G_CALLBACK(focus_follow_toggled_lcb), bfwin);
 		if (filebrowser->basedir == NULL) {
 			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(filebrowser->showfulltree), TRUE);
 			gtk_widget_set_sensitive(GTK_WIDGET(filebrowser->showfulltree), FALSE);
@@ -2018,7 +2080,10 @@ void filebrowser_cleanup(Tbfwin *bfwin) {
 			gtk_list_store_clear(GTK_LIST_STORE(FILEBROWSER(bfwin->filebrowser)->store2));
 			g_object_unref(G_OBJECT(FILEBROWSER(bfwin->filebrowser)->store2));
 		}
-		if (FILEBROWSER(bfwin->filebrowser)->basedir) g_free(FILEBROWSER(bfwin->filebrowser)->basedir);
+		if (FILEBROWSER(bfwin->filebrowser)->basedir)
+			g_free(FILEBROWSER(bfwin->filebrowser)->basedir);
+		if (FILEBROWSER(bfwin->filebrowser)->last_opened_dir) 
+			g_free(FILEBROWSER(bfwin->filebrowser)->last_opened_dir);
 		
 		DEBUG_MSG("filebrowser_cleanup, free filebrowser\n");
 		g_free(bfwin->filebrowser);
