@@ -1,7 +1,23 @@
-/* 
+/* Bluefish HTML Editor
  * fref.c - function reference file
- * 
  *
+ * Copyright (C) 2003 Oliver Sessink and Oskar Swida
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
+/*
  * indent --line-length 100 --k-and-r-style --tab-size 4 -bbo --ignore-newlines 
  */
 
@@ -260,6 +276,8 @@ void fref_loader_start_element(GMarkupParseContext * context, const gchar * elem
 	gpointer pomstr;
 	GtkTreeIter iter;
 	GtkTreeRowReference *rref;
+	GtkTreePath *path = NULL;
+	gchar *string = NULL;
 
 	if (user_data == NULL)
 		return;
@@ -281,9 +299,10 @@ void fref_loader_start_element(GMarkupParseContext * context, const gchar * elem
 				info->type = FR_TYPE_GROUP;
 				info->name = g_strdup(g_hash_table_lookup(attrs, "name"));
 				gtk_tree_store_append(aux->store, &iter, &aux->parent);
+				string = g_strdup(g_hash_table_lookup(attrs, "name"));
 				gtk_tree_store_set(aux->store, &iter, STR_COLUMN,
-								   g_strdup(g_hash_table_lookup(attrs, "name")), FILE_COLUMN, NULL,
-								   PTR_COLUMN, info, -1);
+								   string, FILE_COLUMN, NULL, PTR_COLUMN, info, -1);
+				g_free (string);
 				aux->grp_parent[aux->nest_level] = aux->parent;
 				aux->parent = iter;
 			} else
@@ -306,10 +325,8 @@ void fref_loader_start_element(GMarkupParseContext * context, const gchar * elem
 			gtk_tree_store_set(aux->store, &iter, STR_COLUMN, info->name, PTR_COLUMN, info,
 							   FILE_COLUMN, NULL, -1);
 			aux->act_info = info;
-			rref =
-				gtk_tree_row_reference_new(GTK_TREE_MODEL(aux->store),
-										   gtk_tree_model_get_path(GTK_TREE_MODEL(aux->store),
-																   &iter));
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(aux->store), &iter);
+			rref = gtk_tree_row_reference_new(GTK_TREE_MODEL(aux->store), path);
 			g_hash_table_insert(aux->dict, g_strdup(info->name), rref);
 		} else if (strcmp(element_name, "function") == 0) {
 			aux->state = FR_LOADER_STATE_FUNC;
@@ -320,10 +337,8 @@ void fref_loader_start_element(GMarkupParseContext * context, const gchar * elem
 			gtk_tree_store_append(aux->store, &iter, &aux->parent);
 			gtk_tree_store_set(aux->store, &iter, STR_COLUMN, info->name, PTR_COLUMN, info, -1);
 			aux->act_info = info;
-			rref =
-				gtk_tree_row_reference_new(GTK_TREE_MODEL(aux->store),
-										   gtk_tree_model_get_path(GTK_TREE_MODEL(aux->store),
-																   &iter));
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(aux->store), &iter);
+			rref = gtk_tree_row_reference_new(GTK_TREE_MODEL(aux->store), path);
 			g_hash_table_insert(aux->dict, g_strdup(info->name), rref);
 		} else if (strcmp(element_name, "class") == 0) {
 			aux->state = FR_LOADER_STATE_CLASS;
@@ -334,14 +349,12 @@ void fref_loader_start_element(GMarkupParseContext * context, const gchar * elem
 			gtk_tree_store_append(aux->store, &iter, &aux->parent);
 			gtk_tree_store_set(aux->store, &iter, STR_COLUMN, info->name, PTR_COLUMN, info, -1);
 			aux->act_info = info;
-			rref =
-				gtk_tree_row_reference_new(GTK_TREE_MODEL(aux->store),
-										   gtk_tree_model_get_path(GTK_TREE_MODEL(aux->store),
-																   &iter));
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(aux->store), &iter);
+			rref = gtk_tree_row_reference_new(GTK_TREE_MODEL(aux->store), path);
 			g_hash_table_insert(aux->dict, g_strdup(info->name), rref);
 		} else if (strcmp(element_name, "ref") == 0) {
 		} else
-			g_warning("FREF Config Error: Unknown element");
+			g_warning("FREF Config Error: Unknown element %s", element_name);
 		break;					/* state NONE */
 
 
@@ -450,6 +463,7 @@ void fref_loader_start_element(GMarkupParseContext * context, const gchar * elem
 		break;					/* state PARAM */
 	}							/* switch */
 
+	if (path) gtk_tree_path_free (path);
 	g_hash_table_destroy(attrs);
 }
 
@@ -555,6 +569,9 @@ void fref_loader_text(GMarkupParseContext * context, const gchar * _text, gsize
 	case FR_LOADER_STATE_GROUPDESC:
 		gtk_tree_model_get(GTK_TREE_MODEL(aux->store), &aux->parent, PTR_COLUMN, &tmpinfo, -1);
 		if (tmpinfo != NULL) {
+			/* BUG?? this appears as if it is freed in fref_free_info()
+			 * but a small memory leak is shown when opening the HTML ref
+			 */
 			tmpinfo->description = g_strndup(text, text_len);
 		}
 		break;
@@ -744,11 +761,15 @@ void fref_loader_unload_all(GtkWidget * tree, GtkTreeStore * store)
 
 	DEBUG_MSG("fref_loader_unload_all, started for tree=%p, store=%p\n", tree, store);
 	while (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &iter, NULL, 0)) {
-		if (gtk_tree_path_get_depth(gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter)) == 1) {
+		GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
+		if (gtk_tree_path_get_depth(path) == 1) {
 			gchar *cat;
 			gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, STR_COLUMN, &cat, -1);
 			DEBUG_MSG("fref_loader_unload_all, cat=%s\n", cat);
 			aux = g_hash_table_lookup(FREFDATA(main_v->frefdata)->refcount, cat);
+			if (cat) 
+				g_free (cat);
+				
 			if (aux != NULL) {
 				cnt = (gint *) aux;
 				*cnt = (*cnt) - 1;
@@ -758,7 +779,7 @@ void fref_loader_unload_all(GtkWidget * tree, GtkTreeStore * store)
 					do_unload = FALSE;
 				DEBUG_MSG("fref_loader_unload_all, cnt=%d, do_unload=%d\n", *cnt, do_unload);
 			} else
-				do_unload = FALSE;
+				do_unload = FALSE;				
 		} else
 			do_unload = FALSE;
 		if (do_unload) {
@@ -777,6 +798,8 @@ void fref_loader_unload_all(GtkWidget * tree, GtkTreeStore * store)
 			}
 			gtk_tree_store_remove(store, &iter);
 		}						/* do_unload */
+		if (path)
+			gtk_tree_path_free (path);
 	}							/* while */
 	/* gtk_tree_store_clear(store); */
 }
@@ -825,6 +848,8 @@ void fref_cleanup(Tbfwin * bfwin)
 	FREFGUI(bfwin->fref)->tree = NULL;
 	FREFGUI(bfwin->fref)->argtips = NULL;
 	tree_tips_destroy(FREFGUI(bfwin->fref)->tips);
+	g_free (bfwin->fref);
+	bfwin->fref = NULL;
 }
 
 /* fref_init is ONCE called by bluefish.c to init the fref_data structure */
@@ -918,18 +943,24 @@ gchar *fref_prepare_info(FRInfo * entry, gint infotype, gboolean use_colors)
 		case FR_INFO_ATTRS:
 			lst = g_list_first(entry->attributes);
 			while (lst) {
+				gchar *escaped1, *escaped2;
 				tmpa = (FRAttrInfo *) lst->data;
 				tofree = ret;
-				if (use_colors)
+				escaped1 = xml_escape(tmpa->name);
+				escaped2 = xml_escape(tmpa->description);
+				if (use_colors) 
+					
 					ret =
-						g_strconcat(ret, "<span size=\"small\" >          <b><i>", tmpa->name,
+						g_strconcat(ret, "<span size=\"small\" >          <b><i>", escaped1,
 									"</i></b></span> - <span size=\"small\" foreground=\"",
-									FR_COL_3, "\" >", tmpa->description, "</span>\n", NULL);
+									FR_COL_3, "\" >", escaped2, "</span>\n", NULL);
 				else
 					ret =
-						g_strconcat(ret, "<span size=\"small\" >          <b><i>", tmpa->name,
-									"</i></b></span> - <span size=\"small\">", tmpa->description,
+						g_strconcat(ret, "<span size=\"small\" >          <b><i>", escaped1,
+									"</i></b></span> - <span size=\"small\">", escaped2,
 									"</span>\n", NULL);
+				g_free(escaped1);
+				g_free(escaped2);
 				g_free(tofree);
 				lst = g_list_next(lst);
 			}
@@ -1014,25 +1045,40 @@ gchar *fref_prepare_info(FRInfo * entry, gint infotype, gboolean use_colors)
 				tmpp = (FRParamInfo *) lst->data;
 				tofree = ret;
 				if (tmpp->description != NULL && tmpp->type != NULL) {
+					gchar *escaped1, *escaped2, *escaped3;
+					escaped1 = xml_escape(tmpp->name);
+					escaped2 = xml_escape(tmpp->type);
+					escaped3 = xml_escape(tmpp->description);
 					if (use_colors)
 						ret =
-							g_strconcat(ret, "<span size=\"small\">          <b><i>", tmpp->name,
-										"(", tmpp->type,
+							g_strconcat(ret, "<span size=\"small\">          <b><i>", escaped1,
+										"(", escaped2,
 										")</i></b></span> - <span size=\"small\" foreground=\"",
-										FR_COL_3, "\">", tmpp->description, "</span>\n", NULL);
+										FR_COL_3, "\">", escaped3, "</span>\n", NULL);
 					else
 						ret =
-							g_strconcat(ret, "<span size=\"small\">          <b><i>", tmpp->name,
-										"(", tmpp->type, ")</i></b></span> - <span size=\"small\">",
-										tmpp->description, "</span>\n", NULL);
+							g_strconcat(ret, "<span size=\"small\">          <b><i>", escaped1,
+										"(", escaped2, ")</i></b></span> - <span size=\"small\">",
+										escaped3, "</span>\n", NULL);
+					g_free(escaped1);
+					g_free(escaped2);
+					g_free(escaped3);
 				} else if (tmpp->type != NULL) {
+					gchar *escaped1, *escaped2;
+					escaped1 = xml_escape(tmpp->name);
+					escaped2 = xml_escape(tmpp->type);
 					ret =
-						g_strconcat(ret, "<span size=\"small\">          <b><i>", tmpp->name, "(",
-									tmpp->type, ")</i></b></span>\n", NULL);
+						g_strconcat(ret, "<span size=\"small\">          <b><i>", escaped1, "(",
+									escaped2, ")</i></b></span>\n", NULL);
+					g_free(escaped1);
+					g_free(escaped2);
 				} else {
+					gchar *escaped1;
+					escaped1 = xml_escape(tmpp->name);
 					ret =
-						g_strconcat(ret, "<span size=\"small\">          <b><i>", tmpp->name,
+						g_strconcat(ret, "<span size=\"small\">          <b><i>", escaped1,
 									"</i></b></span>\n", NULL);
+					g_free(escaped1);
 				}
 				g_free(tofree);
 				lst = g_list_next(lst);
@@ -1289,7 +1335,8 @@ static void frefcb_row_expanded(GtkTreeView * treeview, GtkTreeIter * arg1, GtkT
 				do_load = FALSE;
 		} else
 			do_load = FALSE;
-		g_free(val);
+		g_value_unset (val);
+		g_free (val);
 	} else
 		do_load = FALSE;
 	if (do_load) {
@@ -1311,7 +1358,8 @@ static void frefcb_row_expanded(GtkTreeView * treeview, GtkTreeIter * arg1, GtkT
 			&& gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(user_data), &iter, arg1, 0)) {
 			gtk_tree_store_remove(GTK_TREE_STORE(user_data), &iter);
 		}
-		g_free(val);
+		g_value_unset (val);
+		g_free (val);
 	}
 }
 
@@ -1915,7 +1963,7 @@ static guint fref_idle_cleanup(Tfref_cleanup * data)
 	DEBUG_MSG("fref_idle_cleanup, started for data=%s\n", data->cat);
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(FREFDATA(main_v->frefdata)->store), &iter);
 	while (cont) {
-		gchar *str;
+		gchar *str = NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(FREFDATA(main_v->frefdata)->store), &iter, STR_COLUMN,
 						   &str, -1);
 		DEBUG_MSG("fref_idle_cleanup, comparing %s,%s\n", str, data->cat);
@@ -1924,13 +1972,15 @@ static guint fref_idle_cleanup(Tfref_cleanup * data)
 			fref_loader_unload_ref(GTK_TREE_STORE(FREFDATA(main_v->frefdata)->store), &iter);
 			break;
 		}
+		g_free (str);
 		cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(FREFDATA(main_v->frefdata)->store), &iter);
 	}
 	{
 		GtkTreeIter newiter;
 		gtk_tree_store_append(GTK_TREE_STORE(FREFDATA(main_v->frefdata)->store), &newiter, &iter);
 	}
-	g_free(data);
+	g_free (data->cat);
+	g_free (data);
 	return FALSE;				/* do not call again */
 }
 
@@ -2101,7 +2151,7 @@ static gboolean frefcb_event_mouseclick(GtkWidget * widget, GdkEventButton * eve
 static void frefcb_cursor_changed(GtkTreeView * treeview, Tbfwin * bfwin)
 {
 	FRInfo *entry;
-	gchar *info = NULL;
+	gchar *info = NULL, *tmpinfo = NULL; 
 	GdkRectangle rect;
 
 	entry = get_current_entry(bfwin);
@@ -2111,21 +2161,17 @@ static void frefcb_cursor_changed(GtkTreeView * treeview, Tbfwin * bfwin)
 		if (entry->description != NULL) {
 			switch (main_v->globses.fref_info_type) {
 			case FREF_IT_DESC:
-				info =
-					g_strconcat("<span size=\"small\"><b>Description:</b></span>\n",
-								fref_prepare_info(entry, FR_INFO_DESC, FALSE), NULL);
+				tmpinfo = fref_prepare_info(entry, FR_INFO_DESC, FALSE);
+				info = g_strconcat("<span size=\"small\"><b>Description:</b></span>\n", tmpinfo, NULL);
 				break;
 			case FREF_IT_ATTRS:
+				tmpinfo = fref_prepare_info(entry, FR_INFO_ATTRS, FALSE);
 				switch (entry->type) {
 				case FR_TYPE_TAG:
-					info =
-						g_strconcat("<span size=\"small\"><b>Attributes:</b></span>\n",
-									fref_prepare_info(entry, FR_INFO_ATTRS, FALSE), NULL);
+					info = g_strconcat("<span size=\"small\"><b>Attributes:</b></span>\n", tmpinfo, NULL);
 					break;
 				case FR_TYPE_FUNCTION:
-					info =
-						g_strconcat("<span size=\"small\"><b>Parameters:</b></span>\n",
-									fref_prepare_info(entry, FR_INFO_ATTRS, FALSE), NULL);
+					info = g_strconcat("<span size=\"small\"><b>Parameters:</b></span>\n", tmpinfo, NULL);
 					break;
 				}
 				break;
@@ -2138,6 +2184,7 @@ static void frefcb_cursor_changed(GtkTreeView * treeview, Tbfwin * bfwin)
 			gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(FREFGUI(bfwin->fref)->tree), &rect);
 			gtk_widget_set_size_request(FREFGUI(bfwin->fref)->infoview, rect.width, -1);
 			gtk_label_set_markup(GTK_LABEL(FREFGUI(bfwin->fref)->infoview), info);
+			if (tmpinfo) g_free (tmpinfo);
 			g_free(info);
 		} else
 			gtk_label_set_text(GTK_LABEL(FREFGUI(bfwin->fref)->infoview), "");
@@ -2246,6 +2293,7 @@ GtkWidget *fref_gui(Tbfwin * bfwin)
 
 	FREFGUI(bfwin->fref)->tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(fdata->store));
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(FREFGUI(bfwin->fref)->tree), FALSE);
+	/*gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (FREFGUI(bfwin->fref)->tree), TRUE);*/
 	cell = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes("", cell, "text", STR_COLUMN, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(FREFGUI(bfwin->fref)->tree), column);
@@ -2265,7 +2313,9 @@ GtkWidget *fref_gui(Tbfwin * bfwin)
 	gtk_widget_show(FREFGUI(bfwin->fref)->tree);
 	gtk_widget_show(scroll);
 
-	FREFGUI(bfwin->fref)->argtips = gtk_tooltips_new();
+	/* BUG??: don't we have a global tooltips object somewhere ? 
+	FREFGUI(bfwin->fref)->argtips = gtk_tooltips_new();*/
+	FREFGUI(bfwin->fref)->argtips = main_v->tooltips;
 	FREFGUI(bfwin->fref)->tips =
 		tree_tips_new_full(bfwin, GTK_TREE_VIEW(FREFGUI(bfwin->fref)->tree), fref_tip);
 	tree_tips_set_hide_interval(FREFGUI(bfwin->fref)->tips, 5000);
@@ -2285,13 +2335,14 @@ GtkWidget *fref_gui(Tbfwin * bfwin)
 	g_signal_connect(G_OBJECT(FREFGUI(bfwin->fref)->infocheck), "toggled",
 					 G_CALLBACK(frefcb_infocheck_toggled), bfwin);
 	{
+		/* BUG??: is this ever freed ??? */
 		Tcallbackdata *cd = g_new(Tcallbackdata, 1);
 		cd->data = NULL;
 		cd->bfwin = bfwin;
-		btn1 = bf_generic_button_with_image(NULL, 108, G_CALLBACK(frefcb_info_dialog), cd);
+		btn1 = bf_allbuttons_backend(NULL, FALSE, 108, G_CALLBACK(frefcb_info_dialog), cd);
 	}
-	btn2 = bf_generic_button_with_image(NULL, 107, G_CALLBACK(frefcb_full_info), bfwin);
-	btn3 = bf_generic_button_with_image(NULL, 109, G_CALLBACK(frefcb_search), bfwin);
+	btn2 = bf_allbuttons_backend(NULL, FALSE, 107, G_CALLBACK(frefcb_full_info), bfwin);
+	btn3 = bf_allbuttons_backend(NULL, FALSE, 109, G_CALLBACK(frefcb_search), bfwin);
 	gtk_tooltips_set_tip(FREFGUI(bfwin->fref)->argtips, btn1, _("Dialog"), "");
 	gtk_tooltips_set_tip(FREFGUI(bfwin->fref)->argtips, btn2, _("Info"), "");
 	gtk_tooltips_set_tip(FREFGUI(bfwin->fref)->argtips, btn3, _("Search"), "");
