@@ -1,9 +1,9 @@
 /* Bluefish HTML Editor
  * html.c - menu/toolbar callbacks, inserting functions, and other cool stuff 
  *
- * Copyright (C) 
- * 1998 Olivier Sessink and Chris Mazuc
- * 1999-2005 Olivier Sessink
+ * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
+ * modifications Copyright (C) 1999 Antti-Juhani Kaijanaho
+ * Copyright (C) 1999-2005 Olivier Sessink
  * rewrite November 2000 (C) Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,12 +18,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-/* 
- * Changes by Antti-Juhani Kaijanaho <gaia@iki.fi> on 1999-10-20
- * $Id: html.c,v 1.44.2.3 2005/07/01 13:37:24 dleidert Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 /*#define DEBUG*/
 
 #include <gtk/gtk.h>
@@ -325,8 +322,8 @@ static gchar *extract_time_string(char *original_string)
 
 /* time insert struct */
 typedef struct {
-	GtkWidget *check[6];
-	GtkWidget *label[6];
+	GtkWidget *check[7];
+	GtkWidget *label[7];
 	GtkWidget *dialog;
 	Tbfwin *bfwin;
 } TimeInsert;
@@ -347,7 +344,7 @@ static void insert_time_callback(GtkWidget * widget, TimeInsert * timeinsert)
 
 	insert_string = g_malloc0(32);
 	final_string = g_malloc0(255);
-	for (count = 1; count < 6; count++) {
+	for (count = 1; count < 7; count++) {
 		if (GTK_TOGGLE_BUTTON(timeinsert->check[count])->active) {
 			gtk_label_get(GTK_LABEL(timeinsert->label[count]), &temp_string);
 			insert_string = extract_time_string(temp_string);
@@ -378,6 +375,7 @@ static void insert_time_cancel(GtkWidget * widget, TimeInsert * data)
 void insert_time_dialog(Tbfwin *bfwin) {
 
 	gint month, year, count;
+	gchar isotime[60];
 	time_t time_var;
 	gchar *temp = NULL;
 	struct tm *time_struct;
@@ -394,7 +392,7 @@ void insert_time_dialog(Tbfwin *bfwin) {
 	vbox = gtk_vbox_new(FALSE, 1);
 	gtk_container_add(GTK_CONTAINER(timeinsert->dialog), vbox);
 
-	for (count = 1; count < 6; count++) {
+	for (count = 1; count < 7; count++) {
 		switch (count) {
 		case 1:
 			temp = g_strdup_printf(_("  _Time (%i:%i:%i)"), time_struct->tm_hour, time_struct->tm_min, time_struct->tm_sec);
@@ -441,6 +439,10 @@ void insert_time_dialog(Tbfwin *bfwin) {
 			/* Replace \n on ')' */
 			temp[strlen(temp) - 1] = ')';
 			break;
+		case 6:
+			strftime(isotime, 30, "%Y-%m-%dT%H:%M:%S%z", time_struct);
+			temp = g_strdup_printf(_("  ISO-8601 Ti_me (%s)"), isotime);
+			break;
 		default:
 			break;
 		}						/* end of switch count */
@@ -1132,6 +1134,9 @@ void quickstart_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 		tmpstr = g_strconcat("<meta name=\"author\" content=\"\"",endstr,NULL);
 		recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
 		g_free(tmpstr);
+		tmpstr = g_strconcat("<meta name=\"date\" content=\"\"",endstr,NULL);
+		recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
+		g_free(tmpstr);
 		tmpstr = g_strconcat("<meta name=\"copyright\" content=\"\"",endstr,NULL);
 		recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
 		g_free(tmpstr);
@@ -1144,15 +1149,15 @@ void quickstart_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 		tmpstr = g_strconcat("<meta name=\"ROBOTS\" content=\"NOINDEX, NOFOLLOW\"", endstr, NULL);
 		recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
 		g_free(tmpstr);		
-		tmpstr = g_strconcat("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=",main_v->props.newfile_default_encoding,"\"",endstr,NULL);
+		tmpstr = g_strconcat("<meta http-equiv=\"content-type\" content=\"text/html; charset=",main_v->props.newfile_default_encoding,"\"",endstr,NULL);
 		recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
 		g_free(tmpstr);
 		if (main_v->props.xhtml == 1) {
-			tmpstr = g_strconcat("<meta http-equiv=\"Content-Type\" content=\"application/xhtml+xml; charset=",main_v->props.newfile_default_encoding,"\"",endstr,NULL);			
+			tmpstr = g_strconcat("<meta http-equiv=\"content-type\" content=\"application/xhtml+xml; charset=",main_v->props.newfile_default_encoding,"\"",endstr,NULL);			
 			recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
 			g_free(tmpstr);
 		}
-		tmpstr = g_strconcat("<meta http-equiv=\"Expires\" content=\"Tue, 20 Aug 2004 14:25:27 GMT\"", endstr, NULL);
+		tmpstr = g_strconcat("<meta http-equiv=\"expires\" content=\"0\"", endstr, NULL);
 		recent_attribs.headerlist = add_to_stringlist(recent_attribs.headerlist, tmpstr);
 		g_free(tmpstr);
 		tmpstr = g_strconcat("<meta http-equiv=\"refresh\" content=\"5; URL=http://\"", endstr, NULL);
@@ -1223,8 +1228,8 @@ static void bodyok_lcb(GtkWidget * widget, Thtml_diag *dg) {
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[6])->entry), cap("CLASS"), thestring, NULL);
 	thestring = insert_string_if_entry(dg->entry[4], cap("ID"), thestring, NULL);
 	thestring = insert_string_if_entry(dg->entry[5], cap("LANG"), thestring, NULL);
-	thestring = insert_string_if_entry(dg->entry[6], cap("OnLoad"), thestring, NULL);
-	thestring = insert_string_if_entry(dg->entry[7], cap("OnUnLoad"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[6], cap("ONLOAD"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[7], cap("ONUNLOAD"), thestring, NULL);
 
 	thestring = insert_string_if_entry(dg->entry[2], NULL, thestring, NULL);
 	finalstring = g_strconcat(thestring, ">\n", NULL);
@@ -1386,8 +1391,12 @@ static void metaok_lcb(GtkWidget * widget, Thtml_diag *dg)
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[1])->entry), cap("HTTP-EQUIV"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->combo[2])->entry), cap("NAME"), thestring, NULL);
 	thestring = insert_string_if_entry(dg->entry[1], cap("CONTENT"), thestring, NULL);
-	thestring = insert_string_if_entry(dg->entry[2], cap("SCHEME"), thestring, NULL);
-	thestring = insert_string_if_entry(dg->entry[3], NULL, thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[2], cap("LANG"), thestring, NULL);
+	if (main_v->props.xhtml == 1) {
+		thestring = insert_string_if_entry(dg->entry[2], cap("XML:LANG"), thestring, NULL);
+	}
+	thestring = insert_string_if_entry(dg->entry[3], cap("SCHEME"), thestring, NULL);
+	thestring = insert_string_if_entry(dg->entry[4], NULL, thestring, NULL);
 	finalstring = g_strconcat(thestring,main_v->props.xhtml == 1 ? " />" : ">", NULL);
 	g_free(thestring);
 
@@ -1404,8 +1413,8 @@ static void metaok_lcb(GtkWidget * widget, Thtml_diag *dg)
 void meta_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 	GList *popuplist = NULL;
 
-	static gchar *tagitems[] = { "http-equiv", "name", "content", "scheme", NULL };
-	gchar *tagvalues[5];
+	static gchar *tagitems[] = { "http-equiv", "name", "content", "lang", "scheme", NULL };
+	gchar *tagvalues[6];
 	gchar *custom = NULL;
 	Thtml_diag *dg;
 	GtkWidget *dgtable;
@@ -1415,25 +1424,38 @@ void meta_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 
 	dgtable = html_diag_table_in_vbox(dg, 5, 10);
 
-	popuplist = g_list_append(NULL, "Description");
-	popuplist = g_list_append(popuplist, "Date");
-	popuplist = g_list_append(popuplist, "Copyright");
-	popuplist = g_list_append(popuplist, "Author");
-	popuplist = g_list_append(popuplist, "Reply-to"); /* name or equiv? */
-	popuplist = g_list_append(popuplist, "Keywords");
-	popuplist = g_list_append(popuplist, "PICS-label"); /* name or equiv? */
-	popuplist = g_list_append(popuplist, "Refresh");
+	popuplist = g_list_append(NULL, "abstract");
+	popuplist = g_list_append(popuplist, "audience");
+	popuplist = g_list_append(popuplist, "author");
+	popuplist = g_list_append(popuplist, "copyright");
+	popuplist = g_list_append(popuplist, "date");
+	popuplist = g_list_append(popuplist, "description");
+	popuplist = g_list_append(popuplist, "keywords");
+	popuplist = g_list_append(popuplist, "page-topic");
+	popuplist = g_list_append(popuplist, "page-type");
+	popuplist = g_list_append(popuplist, "publisher");
+	popuplist = g_list_append(popuplist, "revisit-after");
+	popuplist = g_list_append(popuplist, "robots");
 	dg->combo[2] = combo_with_popdown(tagvalues[1], popuplist, 1);
 	bf_mnemonic_label_tad_with_alignment(_("_Name:"), dg->combo[2], 0, 0.5, dgtable, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[2])), 1, 10, 0, 1);
 	g_list_free(popuplist);
 
-	popuplist = g_list_append(NULL, "Expires");
-	popuplist = g_list_append(popuplist, "Content-Type");
-	popuplist = g_list_append(popuplist, "Content-Location");
-	popuplist = g_list_append(popuplist, "Content-Language");
-	popuplist = g_list_append(popuplist, "Content-Encoding");
-	popuplist = g_list_append(popuplist, "Pragma");
+	/* extra field for Dublin Core meta tags: DC.foo ?*/
+	
+	popuplist = g_list_append(NULL, "expires");
+	popuplist = g_list_append(popuplist, "refresh");
+	popuplist = g_list_append(popuplist, "content-encoding");
+	popuplist = g_list_append(popuplist, "content-location");
+	popuplist = g_list_append(popuplist, "content-language");
+	popuplist = g_list_append(popuplist, "content-style-type");
+	popuplist = g_list_append(popuplist, "content-script-type");
+	popuplist = g_list_append(popuplist, "content-type");
+	popuplist = g_list_append(popuplist, "ext-cache");
+	popuplist = g_list_append(popuplist, "cache-control");
+	popuplist = g_list_append(popuplist, "pragma");
+	popuplist = g_list_append(popuplist, "set-cookie");
+	popuplist = g_list_append(popuplist, "PICS-Label");
 	dg->combo[1] = combo_with_popdown(tagvalues[0], popuplist, 1);
 	bf_mnemonic_label_tad_with_alignment(_("_HTTP-EQUIV:"), dg->combo[1], 0, 0.5, dgtable, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_COMBO(dg->combo[1])), 1, 10, 1, 2);
@@ -1444,13 +1466,17 @@ void meta_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 10, 2, 3);
 
 	dg->entry[2] = entry_with_text(tagvalues[3], 256);
-	bf_mnemonic_label_tad_with_alignment(_("_Scheme:"), dg->entry[2], 0, 0.5, dgtable, 0, 1, 3, 4);
+	bf_mnemonic_label_tad_with_alignment(_("_Language:"), dg->entry[2], 0, 0.5, dgtable, 0, 1, 3, 4);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[2], 1, 10, 3, 4);
-
-	dg->entry[3] = entry_with_text(custom, 1024);
-	bf_mnemonic_label_tad_with_alignment(_("Custo_m:"), dg->entry[3], 0, 0.5, dgtable, 0, 1, 4, 5);
+	
+	dg->entry[3] = entry_with_text(tagvalues[4], 256);
+	bf_mnemonic_label_tad_with_alignment(_("_Scheme:"), dg->entry[3], 0, 0.5, dgtable, 0, 1, 4, 5);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[3], 1, 10, 4, 5);
 
+	dg->entry[4] = entry_with_text(custom, 1024);
+	bf_mnemonic_label_tad_with_alignment(_("Custo_m:"), dg->entry[4], 0, 0.5, dgtable, 0, 1, 5, 6);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[4], 1, 10, 5, 6);
+
 	html_diag_finish(dg, G_CALLBACK(metaok_lcb));
 	if (custom)	g_free(custom);
 }
@@ -1497,6 +1523,8 @@ static void generalfontdialog_lcb(gint type, GtkWidget * widget, Thtml_diag *dg)
 	finalstring = g_strconcat(thestring, ">", NULL);
 	g_free(thestring);
 
+	dg->bfwin->session->fontlist = add_entry_to_stringlist(dg->bfwin->session->fontlist, GTK_WIDGET(GTK_COMBO(dg->combo[3])->entry));
+
 	if (dg->range.end == -1) {
 		doc_insert_two_strings(dg->doc, finalstring, cap("</FONT>"));
 	} else {
@@ -1584,32 +1612,77 @@ void basefont_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 
 static void emailok_lcb(GtkWidget * widget, Thtml_diag *dg)
 {
-	gchar *finalstring, *tmpstr, *subj, *body;
+	gchar *finalstring, *tmpstr, *cc, *bcc, *subj, *body;
+	gchar *urlencoded = NULL;
 	gboolean have_questionmark = FALSE;
 	tmpstr = g_strconcat(cap("<A HREF=\"mailto:")
 			, gtk_entry_get_text(GTK_ENTRY(dg->entry[1]))
 			, NULL);
 	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[2])))) {
-		subj = g_strconcat("?Subject=", gtk_entry_get_text(GTK_ENTRY(dg->entry[2])), NULL);
+		cc = g_strconcat("?cc=", gtk_entry_get_text(GTK_ENTRY(dg->entry[2])), NULL);
 		have_questionmark = TRUE;
+	} else {
+		cc = g_strdup("");
+	}
+	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[3])))) {
+		if (have_questionmark) {
+			bcc = g_strconcat("&amp;bcc=", gtk_entry_get_text(GTK_ENTRY(dg->entry[3])), NULL);
+		} else {
+			bcc = g_strconcat("?bcc=", gtk_entry_get_text(GTK_ENTRY(dg->entry[3])), NULL);
+			have_questionmark = TRUE;
+		}		
+	} else {
+		bcc = g_strdup("");
+	}
+	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[4])))) {
+#ifdef HAVE_GNOME_VFS
+		urlencoded = gnome_vfs_escape_string(gtk_entry_get_text(GTK_ENTRY(dg->entry[4])));
+#endif /* HAVE_GNOME_VFS */
+		if (have_questionmark) {
+#ifdef HAVE_GNOME_VFS
+			subj = g_strconcat("&amp;subject=", urlencoded, NULL);
+#else
+			subj = g_strconcat("&amp;subject=", gtk_entry_get_text(GTK_ENTRY(dg->entry[4])), NULL);
+#endif /* HAVE_GNOME_VFS */
+		} else {
+#ifdef HAVE_GNOME_VFS
+			subj = g_strconcat("?subject=", urlencoded, NULL);
+#else
+			subj = g_strconcat("?subject=", gtk_entry_get_text(GTK_ENTRY(dg->entry[4])), NULL);
+#endif /* HAVE_GNOME_VFS */
+			have_questionmark = TRUE;			
+		}		
 	} else {
 		subj = g_strdup("");
 	}
-	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[3])))) {
-		gchar *str2;
+	if (strlen(gtk_entry_get_text(GTK_ENTRY(dg->entry[5])))) {
+#ifdef HAVE_GNOME_VFS
+		urlencoded = gnome_vfs_escape_string(gtk_entry_get_text(GTK_ENTRY(dg->entry[5])));
+#endif /* HAVE_GNOME_VFS */
 		if (have_questionmark) {
-			str2 = "&Body=";
+#ifdef HAVE_GNOME_VFS
+			body = g_strconcat("&amp;body=", urlencoded, NULL);
+#else
+			body = g_strconcat("&amp;body=", gtk_entry_get_text(GTK_ENTRY(dg->entry[5])), NULL);
+#endif /* HAVE_GNOME_VFS */
 		} else {
-			str2 = "?Body=";
-		}
-		body = g_strconcat(str2, gtk_entry_get_text(GTK_ENTRY(dg->entry[3])), NULL);
+#ifdef HAVE_GNOME_VFS
+			body = g_strconcat("?body=", urlencoded, NULL);
+#else
+			body = g_strconcat("?body=", gtk_entry_get_text(GTK_ENTRY(dg->entry[5])), NULL);
+#endif /* HAVE_GNOME_VFS */
+			have_questionmark = TRUE;			
+		}		
 	} else {
 		body = g_strdup("");
 	}
-	finalstring = g_strconcat(tmpstr, subj, body, "\">", NULL);
+	finalstring = g_strconcat(tmpstr, cc, bcc, subj, body, "\">", NULL);
 	g_free(tmpstr);
+	g_free(cc);
+	g_free(bcc);
 	g_free(subj);
 	g_free(body);
+	g_free(urlencoded);
 	doc_insert_two_strings(dg->doc, finalstring, cap("</A>"));
 	g_free(finalstring);
 	html_diag_destroy_cb(NULL, dg);
@@ -1619,25 +1692,40 @@ void email_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 	GtkWidget *dgtable;
 	Thtml_diag *dg;
 	
-	dg = html_diag_new(bfwin,_("Email"));
+	dg = html_diag_new(bfwin,_("E-mail"));
 
-	dgtable = gtk_table_new(3, 2, FALSE);
+	dgtable = gtk_table_new(5, 2, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(dgtable), 12);
 	gtk_table_set_row_spacings(GTK_TABLE(dgtable), 12);
 	gtk_box_pack_start(GTK_BOX(dg->vbox), dgtable, FALSE, FALSE, 0);
 
 	dg->entry[1] = gtk_entry_new_with_max_length(256);
-	bf_mnemonic_label_tad_with_alignment(_("_Email address:"), dg->entry[1], 0, 0.5, dgtable, 0, 1, 0, 1);
+	bf_mnemonic_label_tad_with_alignment(_("_Mail to:"), dg->entry[1], 0, 0.5, dgtable, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[1], 1, 2, 0, 1);
 
-	
 	dg->entry[2] = gtk_entry_new_with_max_length(256);
-	bf_mnemonic_label_tad_with_alignment(_("UrlEncoded _subject:"), dg->entry[2], 0, 0.5, dgtable, 0, 1, 1, 2);
+	bf_mnemonic_label_tad_with_alignment(_("_Copy to:"), dg->entry[2], 0, 0.5, dgtable, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[2], 1, 2, 1, 2);
 
 	dg->entry[3] = gtk_entry_new_with_max_length(256);
-	bf_mnemonic_label_tad_with_alignment(_("UrlEncoded _body:"), dg->entry[3], 0, 0.5, dgtable, 0, 1, 2, 3);
+	bf_mnemonic_label_tad_with_alignment(_("Blin_d copy to:"), dg->entry[3], 0, 0.5, dgtable, 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[3], 1, 2, 2, 3);
+	
+	dg->entry[4] = gtk_entry_new_with_max_length(256);
+#ifdef HAVE_GNOME_VFS
+	bf_mnemonic_label_tad_with_alignment(_("with _Subject:"), dg->entry[4], 0, 0.5, dgtable, 0, 1, 3, 4);
+#else
+	bf_mnemonic_label_tad_with_alignment(_("with _Subject (URL-encoded):"), dg->entry[4], 0, 0.5, dgtable, 0, 1, 3, 4);
+#endif /* HAVE_GNOME_VFS */
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[4], 1, 2, 3, 4);
+
+	dg->entry[5] = gtk_entry_new_with_max_length(256);
+#ifdef HAVE_GNOME_VFS
+	bf_mnemonic_label_tad_with_alignment(_("with _Body:"), dg->entry[5], 0, 0.5, dgtable, 0, 1, 4, 5);
+#else
+	bf_mnemonic_label_tad_with_alignment(_("with _Body (URL-encoded):"), dg->entry[5], 0, 0.5, dgtable, 0, 1, 4, 5);
+#endif /* HAVE_GNOME_VFS */
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->entry[5], 1, 2, 4, 5);
 
 	html_diag_finish(dg, G_CALLBACK(emailok_lcb));
 }
@@ -1907,7 +1995,7 @@ static void embedok_lcb(GtkWidget * widget,Thtml_diag *dg )
 	thestring = g_strdup(cap("<EMBED"));
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->entry[1])), cap("SRC"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[1])), cap("WIDTH"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[2])), cap("HEGHT"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[2])), cap("HEIGHT"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(dg->spin[3])), cap("BORDER"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_ENTRY(GTK_COMBO(dg->combo[1])->entry)), cap("ALIGN"), thestring, NULL);
 
@@ -2033,11 +2121,13 @@ static void linkdialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
 	thestring = g_strdup(cap("<LINK"));
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[0])->entry), cap("HREF"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[1]), cap("HREFLANG"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[2])->entry), cap("TYPE"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[3])->entry), cap("REL"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[4])->entry), cap("REV"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[5])->entry), cap("MEDIA"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[6]), cap("LANG"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[2]), cap("TITLE"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[3])->entry), cap("TYPE"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[4])->entry), cap("REL"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[5])->entry), cap("REV"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(dg->attrwidget[6])->entry), cap("MEDIA"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[7]), cap("LANG"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_WIDGET(dg->attrwidget[8]), NULL, thestring, NULL);
 	finalstring = g_strconcat(thestring, (main_v->props.xhtml == 1) ? " />" : ">", NULL);
 	g_free(thestring);
 
@@ -2050,9 +2140,9 @@ static void linkdialogok_lcb(GtkWidget * widget, Thtml_diag *dg) {
 	html_diag_destroy_cb(NULL, dg);
 }
 
-void linkdialog_dialog(Tbfwin *bfwin, Ttagpopup *data) {
-	static gchar *tagitems[] = {"href", "hreflang", "type", "rel", "rev", "media", "lang", NULL };
-	gchar *tagvalues[8];
+void linkdialog_dialog(Tbfwin *bfwin, Ttagpopup *data, Tlinkdialog_mode mode) {
+	static gchar *tagitems[] = {"href", "hreflang", "title", "type", "rel", "rev", "media", "lang", NULL };
+	gchar *tagvalues[9];
 	gchar *custom = NULL;
 	Thtml_diag *dg;
 	GtkWidget *dgtable, *but;
@@ -2068,52 +2158,83 @@ void linkdialog_dialog(Tbfwin *bfwin, Ttagpopup *data) {
 		dg->attrwidget[0] = combo_with_popdown(tagvalues[0], rel_link_list, 1);
 		free_stringlist(rel_link_list);
 	}
-	bf_mnemonic_label_tad_with_alignment(_("_Href:"), dg->attrwidget[0], 0, 0.5, dgtable, 0, 1, 0, 1);
+	bf_mnemonic_label_tad_with_alignment(_("_HREF:"), dg->attrwidget[0], 0, 0.5, dgtable, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[0], 1, 2, 0, 1);
 	but = file_but_new(GTK_COMBO(dg->attrwidget[0])->entry, 0, bfwin);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), but, 2, 3, 0, 1);
 
 	dg->attrwidget[1] = entry_with_text(tagvalues[1], 1024);
-	bf_mnemonic_label_tad_with_alignment(_("Href_lang:"), dg->attrwidget[1], 0, 0.5, dgtable, 0, 1, 1, 2);
+	bf_mnemonic_label_tad_with_alignment(_("HREF_LANG:"), dg->attrwidget[1], 0, 0.5, dgtable, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[1], 1, 3, 1, 2);
 
-	{
-		GList *tmplist = list_from_arglist(FALSE, "text/html", "text/css", "text/plain", "text/javascript", "application/postscript", NULL);
-		dg->attrwidget[2] = combo_with_popdown(tagvalues[2], tmplist, 1);
-		g_list_free(tmplist);
-	}
-	bf_mnemonic_label_tad_with_alignment(_("_Type:"), dg->attrwidget[2], 0, 0.5, dgtable, 0, 1, 2, 3);
+	dg->attrwidget[2] = entry_with_text(tagvalues[2], 1024);
+	bf_mnemonic_label_tad_with_alignment(_("T_itle:"), dg->attrwidget[2], 0, 0.5, dgtable, 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[2], 1, 3, 2, 3);
 	
 	{
-		GList *tmplist = list_from_arglist(FALSE, "Alternate", "Stylesheet", "Start", "Next", "Prev", "Contents", "Index", "Glossary", "Copyright", "Chapter", "Section", "Subsection", "Appendix", "Help", "Bookmark", NULL);
-		dg->attrwidget[3] = combo_with_popdown(tagvalues[3], tmplist, 1);
-		bf_mnemonic_label_tad_with_alignment(_("_Forward Relation:"), dg->attrwidget[3], 0, 0.5, dgtable, 0, 1, 3, 4);
-		gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[3], 1, 3, 3, 4);
+		gchar *tmp;
+		GList *tmplist = list_from_arglist(FALSE, "text/html", "text/css", "text/plain", "text/javascript", "application/postscript", NULL);
 		
-		dg->attrwidget[4] = combo_with_popdown(tagvalues[4], tmplist, 1);
-		bf_mnemonic_label_tad_with_alignment(_("_Reverse Relation:"), dg->attrwidget[4], 0, 0.5, dgtable, 0, 1, 4, 5);
+		switch (mode) {
+			case linkdialog_mode_default:
+				tmp = tagvalues[3];
+			break;
+			case linkdialog_mode_css:
+				tmp = "text/css";
+			break;
+			default:
+				tmp = tagvalues[3];
+			break;				
+		}
+		dg->attrwidget[3] = combo_with_popdown(tmp, tmplist, 1);
+
+		g_list_free(tmplist);
+	}
+	bf_mnemonic_label_tad_with_alignment(_("_Type:"), dg->attrwidget[3], 0, 0.5, dgtable, 0, 1, 3, 4);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[3], 1, 3, 3, 4);
+	
+	{
+		gchar *tmp;
+		GList *tmplist = list_from_arglist(FALSE, "stylesheet", "alternate", "alternate stylesheet", "shortcut icon", "contents", "chapter", "section", "subsection", "index", "glossary", "appendix", "search", "author", "copyright", "next", "prev", "first", "last", "up", "top", "help", "bookmark", NULL);
+
+		switch (mode) {
+			case linkdialog_mode_default:
+				tmp = tagvalues[4];
+			break;
+			case linkdialog_mode_css:
+				tmp = "stylesheet";
+			break;
+			default:
+				tmp = tagvalues[4];
+			break;				
+		}
+		dg->attrwidget[4] = combo_with_popdown(tmp, tmplist, 1);
+		bf_mnemonic_label_tad_with_alignment(_("_Forward Relation:"), dg->attrwidget[4], 0, 0.5, dgtable, 0, 1, 4, 5);
 		gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[4], 1, 3, 4, 5);
 		
+		dg->attrwidget[5] = combo_with_popdown(tagvalues[5], tmplist, 1);
+		bf_mnemonic_label_tad_with_alignment(_("_Reverse Relation:"), dg->attrwidget[5], 0, 0.5, dgtable, 0, 1, 5, 6);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[5], 1, 3, 5, 6);
+		
 		g_list_free(tmplist);
 	}
 	
 	{
 		GList *tmplist = list_from_arglist(FALSE, "screen", "tty", "tv", "projection", "handheld", "print", "braille", "aural", "all", NULL);
-		dg->attrwidget[5] = combo_with_popdown(tagvalues[5], tmplist, 1);
+		dg->attrwidget[6] = combo_with_popdown(tagvalues[6], tmplist, 1);
 		g_list_free(tmplist);
 	}
-	bf_mnemonic_label_tad_with_alignment(_("Media:"), dg->attrwidget[5], 0, 0.5, dgtable, 0, 1, 5, 6);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[5], 1, 3, 5, 6);
-
-	dg->attrwidget[6] = entry_with_text(tagvalues[6], 1024);
-	bf_mnemonic_label_tad_with_alignment(_("L_ang:"), dg->attrwidget[6], 0, 0.5, dgtable, 0, 1, 6, 7);
+	bf_mnemonic_label_tad_with_alignment(_("Media:"), dg->attrwidget[6], 0, 0.5, dgtable, 0, 1, 6, 7);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[6], 1, 3, 6, 7);
-	
-	dg->attrwidget[7] = entry_with_text(custom, 1024);
-	bf_mnemonic_label_tad_with_alignment(_("_Custom:"), dg->attrwidget[7], 0, 0.5, dgtable, 0, 1, 7, 8);
+
+	dg->attrwidget[7] = entry_with_text(tagvalues[7], 1024);
+	bf_mnemonic_label_tad_with_alignment(_("L_ang:"), dg->attrwidget[7], 0, 0.5, dgtable, 0, 1, 7, 8);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[7], 1, 3, 7, 8);
 	
+	dg->attrwidget[8] = entry_with_text(custom, 1024);
+	bf_mnemonic_label_tad_with_alignment(_("_Custom:"), dg->attrwidget[8], 0, 0.5, dgtable, 0, 1, 8, 9);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->attrwidget[8], 1, 3, 8, 9);
+	
 	html_diag_finish(dg, G_CALLBACK(linkdialogok_lcb));
 	if (custom)	g_free(custom);
 }	
