@@ -1,5 +1,5 @@
 /* Bluefish HTML Editor
- * quickstart.c --> quickstart dialog
+ * quickstart.c - quickstart dialog
  *
  * Copyright (C) 2005 James Hayward and Olivier Sessink
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
  
 /*
@@ -72,6 +72,7 @@ typedef struct {
 	GtkWidget *scriptsrc;
 	GtkWidget *scriptarea;
 	GtkWidget *removeButton;
+	GtkWidget *openNewDoc;
 	Tbfwin *bfwin;
 } TQuickStart;
 
@@ -179,14 +180,15 @@ quickstart_load_metatags(GtkListStore *lstore)
 	const gchar *metaTags[] = {
 		"name=\"generator\" content=\"Bluefish\"",
 		"name=\"author\" content=\"\"",
+		"name=\"date\" content=\"\"",
 		"name=\"copyright\" content=\"\"",
 		"name=\"keywords\" content=\"\"",
 		"name=\"description\" content=\"\"",
 		"name=\"ROBOTS\" content=\"NOINDEX, NOFOLLOW\"",
-		"http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"",
-		"http-equiv=\"Content-Type\" content=\"application/xhtml+xml; charset=UTF-8\"",
-		"http-equiv=\"Content-Style-Type\" content=\"text/css\"",
-		"http-equiv=\"Expires\" content=\"\"",
+		"http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\"",
+		"http-equiv=\"content-type\" content=\"application/xhtml+xml; charset=UTF-8\"",
+		"http-equiv=\"content-style-type\" content=\"text/css\"",
+		"http-equiv=\"expires\" content=\"0\"",
 		"http-equiv=\"refresh\" content=\"5; URL=http://\"",
 	};
 
@@ -346,18 +348,21 @@ quickstart_response_lcb(GtkDialog *dialog, gint response, TQuickStart *qstart)
 			styletitle = gtk_editable_get_chars (GTK_EDITABLE (qstart->styletitle), 0, -1);
 			
 			if (strcmp(name, "Linked") == 0) {
-				stylestr = g_string_append (stylestr, "<link rel=stylesheet type=\"text/css\" ");
+			    tmpstr2 = g_strdup_printf ("<link href=\"%s\" rel=\"stylesheet\" type=\"text/css\"", stylehref);
+			    stylestr = g_string_append (stylestr, tmpstr2);
+			    g_free (tmpstr2);
+			    
 				if (strlen(stylemedia) > 0) {
 					if (strlen(styletitle) > 0) {
-						tmpstr2 = g_strdup_printf ("media=\"%s\" href=\"%s\" title=\"%s\">\n", stylemedia, stylehref, styletitle);
+						tmpstr2 = g_strdup_printf (" media=\"%s\" title=\"%s\">\n", stylemedia, styletitle);
 					} else {
-						tmpstr2 = g_strdup_printf ("media=\"%s\" href=\"%s\">\n", stylemedia, stylehref);
+						tmpstr2 = g_strdup_printf (" media=\"%s\">\n", stylemedia);
 					}
 				} else {
 					if (strlen(styletitle) > 0) {
-						tmpstr2 = g_strdup_printf ("href=\"%s\" title=\"%s\">\n", stylehref, styletitle);
+						tmpstr2 = g_strdup_printf (" title=\"%s\">\n", styletitle);
 					} else {
-						tmpstr2 = g_strdup_printf ("href=\"%s\">\n", stylehref);
+					    tmpstr2 = g_strdup (">\n");
 					}
 				}
 			} else {
@@ -399,8 +404,8 @@ quickstart_response_lcb(GtkDialog *dialog, gint response, TQuickStart *qstart)
 		}
 		
 		finalstr = g_strconcat (xmlstr, dtdstr, tmpstr, titlestr, metastr->str, 
-										stylestr->str, stylearea, scriptsrc, scriptarea, 
-										cap("</HEAD>\n"), is_frameset_dtd ? cap("<FRAMESET>\n") : cap("<BODY>\n"), NULL);
+                                    stylestr->str, stylearea, scriptsrc, scriptarea, 
+                                    cap("</HEAD>\n"), is_frameset_dtd ? cap("<FRAMESET>\n") : cap("<BODY>\n"), NULL);
 		
 		g_free (xmlstr);
 		g_free (dtdstr);
@@ -411,20 +416,16 @@ quickstart_response_lcb(GtkDialog *dialog, gint response, TQuickStart *qstart)
 		g_free (stylearea);
 		g_free (scriptsrc);
 		g_free (scriptarea);
-
-		/* I'm wondering if it might be better just to check and see if the 
-		 * current doc is empty or whether it should be an option in the dialog
-		 * to create in a new document.
-		 */		
-		if (!doc_is_empty_non_modified_and_nameless(qstart->bfwin->current_document)) {
+		
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (qstart->openNewDoc))) {
 			Tdocument *doc;
 			doc = doc_new(qstart->bfwin, FALSE);
 			switch_to_document_by_pointer(qstart->bfwin, doc);
 		}
 
 		doc_insert_two_strings(qstart->bfwin->current_document, 
-									  finalstr, 
-									  is_frameset_dtd ? cap("\n</FRAMESET>\n</HTML>") : cap("\n</BODY>\n</HTML>"));
+                                finalstr, 
+                                is_frameset_dtd ? cap("\n</FRAMESET>\n</HTML>") : cap("\n</BODY>\n</HTML>"));
 		g_free (finalstr);
 		
 		doc_set_filetype(qstart->bfwin->current_document, get_filetype_by_name("html"));
@@ -459,7 +460,7 @@ quickstart_meta_page_create(TQuickStart *qstart)
 	qstart->metaView = gtk_tree_view_new ();
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (qstart->metaView), FALSE);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (qstart->metaView));
-	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (quickstart_meta_selection_changed), qstart);
+	
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	gtk_container_add (GTK_CONTAINER (scrolwin), qstart->metaView);
 	renderer = gtk_cell_renderer_text_new ();
@@ -482,6 +483,8 @@ quickstart_meta_page_create(TQuickStart *qstart)
 	gtk_widget_set_sensitive (qstart->removeButton, FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 0);
 	
+	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (quickstart_meta_selection_changed), qstart);
+	
 	return hbox;
 }
 
@@ -535,7 +538,7 @@ quickstart_style_page_create(TQuickStart *qstart)
 	
 	qstart->stylehref = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL (history), 0);
 	g_object_unref (history);
-	dialog_mnemonic_label_in_table(_("Hre_f:"), qstart->stylehref, table, 0, 1, 0, 1);
+	dialog_mnemonic_label_in_table(_("HRE_F:"), qstart->stylehref, table, 0, 1, 0, 1);
 	gtk_table_attach (GTK_TABLE (table), qstart->stylehref, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 0);
 	
 	qstart->stylemedia = gtk_entry_new ();
@@ -611,7 +614,7 @@ void
 quickstart_dialog_new(Tbfwin *bfwin)
 {
 	TQuickStart *qstart;
-	GtkWidget *dialog, *table, *frame, *page;
+	GtkWidget *dialog, *table, *frame, *page, *alignment;
 	GtkListStore *headStore;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -635,7 +638,7 @@ quickstart_dialog_new(Tbfwin *bfwin)
 													  NULL);	  
 	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (quickstart_response_lcb), qstart);
 
-	table = dialog_table_in_vbox_defaults(5, 3, 6, GTK_DIALOG (dialog)->vbox);
+	table = dialog_table_in_vbox_defaults(4, 3, 6, GTK_DIALOG (dialog)->vbox);
 	
 	qstart->dtd = gtk_combo_box_new_text ();
 	for (i = 0; i < G_N_ELEMENTS (dtds); i++) {
@@ -686,6 +689,16 @@ quickstart_dialog_new(Tbfwin *bfwin)
 	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (headStore), &iter);
 	gtk_tree_selection_select_iter (selection, &iter);
 	
+	qstart->openNewDoc = gtk_check_button_new_with_mnemonic (_("Open in _new document."));
+	if (doc_is_empty_non_modified_and_nameless(bfwin->current_document))
+	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qstart->openNewDoc), FALSE);
+	else
+	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (qstart->openNewDoc), TRUE);	
+	alignment = gtk_alignment_new (0, 0.5, 0, 0);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 8, 4, 0);
+	gtk_container_add (GTK_CONTAINER (alignment), qstart->openNewDoc);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), alignment, FALSE, FALSE, 0);
+	
 	gtk_widget_show_all (dialog);
 }
 
