@@ -1,6 +1,5 @@
 /* Bluefish HTML Editor
- *
- * undo_redo.h -- undo structures and function declarations
+ * undo_redo.h - undo structures and function declarations
  *
  * Copyright (C) 1998-2002 Olivier Sessink
  *
@@ -16,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef __UNDO_REDO_H_
@@ -28,10 +27,12 @@ void doc_unre_init(Tdocument *doc);
 void doc_unre_destroy(Tdocument *doc);
 void doc_unre_clear_all(Tdocument *doc);
 gint doc_undo_op_compare(Tdocument *doc, undo_op_t testfor, gint position);
+gboolean doc_unre_test_last_entry(Tdocument *doc, undo_op_t testfor, gint start, gint end);
 void redo_cb(GtkWidget * widget, Tbfwin *bfwin);
 void undo_cb(GtkWidget * widget, Tbfwin *bfwin);
 void redo_all_cb(GtkWidget * widget, Tbfwin *bfwin);
 void undo_all_cb(GtkWidget * widget, Tbfwin *bfwin);
 gboolean doc_has_undo_list(Tdocument *doc);
 gboolean doc_has_redo_list(Tdocument *doc);
+void doc_unregroup_reset_changed(Tdocument *doc);
 #endif /* __UNDO_REDO_H_ */
